class Stage : public AR_Bitmap
 {
  protected:

   float scroll_x,scroll_y;
   AR_Map *l0,*l1,*lh;                              // Stage layers map
   int data_rows, data_columns, data_items, **data; // Stage data
   string stage_name;

  public:

   Stage(string filename, AR_Display *display) : AR_Bitmap(filename + "bg.png",display)
    {
	 l0 = new AR_Map(filename + "l0.png",display,2); // Set parallax ratio to 2
	 l1 = new AR_Map(filename + "l1.png",display);
	 lh = new AR_Map(filename + "lh.png",display);
	 l0->setScrollXP(&scroll_x); l0->setScrollYP(&scroll_y);
	 l1->setScrollXP(&scroll_x); l1->setScrollYP(&scroll_y);
	 lh->setScrollXP(&scroll_x); lh->setScrollYP(&scroll_y);
	 loadStageData(filename + "data.dat");
     scroll_x = scroll_y = 0;
     scale_x  = display->getWidth(); // Scale background
    }

   ~Stage() { }

   void debug() { l1->debug(); }

   void run()
    {
	 if(!enable) return;

     if(scroll_x < 0)                                       scroll_x = 0;
     if(scroll_x > l1->getMapWidth() - display->getWidth()) scroll_x = l1->getMapWidth() - display->getWidth();

     if(scroll_y < 0)                                       scroll_y = 0;
     if(scroll_y > l1->getMapHeight() - getHeight())        scroll_y = l1->getMapHeight() - getHeight();
	}

   void draw()
    {
	 // Draw here only background
	 if(!enable) return; AR_Bitmap::draw();
	}

   void loadStageData(string filename)
    {
	 string   aux;
     ifstream fd;

     fd.open(filename.c_str());
     if(!fd.is_open()) { AR_Util::errorBox(display->getDisplay(),string("File '") + string(filename) + string("' not found!")); exit(1); }
     fd >> data_rows;        // Number of enemies type
     fd >> data_columns;     // Number of waves in the stage
     fd >> data_items;       // Number of items per enemies
     getline(fd,stage_name); // Stage name (read also white spaces)
     data = (int **)malloc(sizeof(int *)*data_rows);
     for(int i=0;i<data_rows;i++)
      {
	   data[i] = (int *)malloc(sizeof(int)*data_columns*data_items);
       fd >> aux;
       for(int j=0;j<data_columns*data_items;j+=data_items)
        {
         fd >> data[i][j];   // Number of enemies per wave
         fd >> data[i][j+1]; // Shields on/off for the enemies in the wave
	    }
	  }
     fd.close();
    }

   void drawLayer0() { if(!enable) return; l0->draw();        }

   void drawLayer1() { if(!enable) return; l1->draw();        }

   void drawLayerH() { if(!enable) return; lh->draw();        }

   void setScrollX(float scroll_x) { this->scroll_x = scroll_x; }

   void setScrollY(float scroll_y) { this->scroll_y = scroll_y; }

   void setEnableLayer(int n, bool v)
    {
     if(n == 0) l0->setEnable(v);
     if(n == 1) l1->setEnable(v);
     if(n == 2) lh->setEnable(v);     
	}

   int getEnemiesNumber(int enemy_type, int wave) { return data[enemy_type][wave*2];   }

   int getEnemiesShield(int enemy_type, int wave) { return data[enemy_type][wave*2+1]; }

   float getScrollX() { return scroll_x; }

   float getScrollY() { return scroll_y; }

   float *getScrollXP() { return &scroll_x; }
   
   float *getScrollYP() { return &scroll_y; }

   int getTotalEnemies(int current_wave)
    {
	 int total;
	 
	 total = 0;
     for(int i=0;i<N_ENEMIES_TYPE;i++)
      total += getEnemiesNumber(i,current_wave);
     return total;
	}

   int getMapWidth(int n)
    {
     if(n == 0) l0->getMapWidth();
     if(n == 1) l1->getMapWidth();
     if(n == 2) lh->getMapWidth();
    }

   int getMapHeight(int n)
    {
     if(n == 0) l0->getMapHeight();
     if(n == 1) l1->getMapHeight();
     if(n == 2) lh->getMapHeight();
    }

   bool getEnableLayer(int n)
    {
     if(n == 0) return l0->getEnable();
     if(n == 1) return l1->getEnable();
     if(n == 2) return lh->getEnable();
	}

   string getStageName() { return stage_name; }

 };
